@@ -1,10 +1,13 @@
 # frozen_string_literal: true
+
 require 'test_helper'
 
 if defined?(::Rails::Railtie) && defined?(::ActionView)
   class ActionViewExtensionTest < ActionView::TestCase
     setup do
       self.output_buffer = ::ActionView::OutputBuffer.new
+      I18n.available_locales = [:en, :de, :fr]
+      I18n.locale = :en
     end
     teardown do
       User.delete_all
@@ -26,6 +29,13 @@ if defined?(::Rails::Railtie) && defined?(::ActionView)
         end
       end
 
+      test 'allows for overriding params with the :params option' do
+        view.params[:controller], view.params[:action] = 'addresses', 'new'
+        users = User.page(1)
+
+        assert_match '/users?page=2', view.paginate(users, params: { controller: 'users', action: 'index' })
+      end
+
       test 'accepts :theme option' do
         users = User.page(1)
         begin
@@ -118,6 +128,17 @@ if defined?(::Rails::Railtie) && defined?(::ActionView)
         assert_nil view.link_to_previous_page(users, 'Previous', params: {controller: 'users', action: 'index'})
         assert_equal 'At the Beginning', (view.link_to_previous_page(users, 'Previous', params: {controller: 'users', action: 'index'}) do 'At the Beginning' end)
       end
+
+      test '#link_to_previous_page accepts ActionController::Parameters' do
+        users = User.page(3)
+        params = ActionController::Parameters.new(controller: 'users', action: 'index', status: 'active')
+
+        html = view.link_to_previous_page users, 'Previous', params: params
+
+        assert_match(/page=2/, html)
+        assert_match(/rel="prev"/, html)
+        assert_match(/status=active/, html)
+      end
     end
 
     sub_test_case '#link_to_next_page' do
@@ -159,6 +180,17 @@ if defined?(::Rails::Railtie) && defined?(::ActionView)
 
         assert_nil view.link_to_next_page(users, 'More', params: {controller: 'users', action: 'index'})
       end
+
+      test '#link_to_next_page accepts ActionController::Parameters' do
+        users = User.page(1)
+        params = ActionController::Parameters.new(controller: 'users', action: 'index', status: 'active')
+
+        html = view.link_to_next_page users, 'More', params: params
+
+        assert_match(/page=2/, html)
+        assert_match(/rel="next"/, html)
+        assert_match(/status=active/, html)
+      end
     end
 
     sub_test_case '#page_entries_info' do
@@ -242,12 +274,26 @@ if defined?(::Rails::Railtie) && defined?(::ActionView)
                 User.max_pages 4
                 users = User.page(4).per(10)
 
-                assert_equal 'Displaying users <b>31&nbsp;-&nbsp;40</b> of <b>50</b> in total', view.page_entries_info(users)
+                assert_equal 'Displaying users <b>31&nbsp;-&nbsp;40</b> of <b>40</b> in total', view.page_entries_info(users)
               ensure
                 User.max_pages nil
               end
             end
           end
+
+          test 'it accepts a decorated object' do
+            page_info_presenter = Class.new(SimpleDelegator) do
+              include ActionView::Helpers::NumberHelper
+
+              def total_count
+                number_with_delimiter(1_000)
+              end
+            end
+
+            users = page_info_presenter.new(User.page(1).per(25))
+
+            assert_equal 'Displaying users <b>1&nbsp;-&nbsp;25</b> of <b>1,000</b> in total', view.page_entries_info(users)
+          end
         end
       end
 
@@ -266,6 +312,90 @@ if defined?(::Rails::Railtie) && defined?(::ActionView)
             I18n.backend.reload!
           end
         end
+
+        sub_test_case 'with any other locale' do
+          teardown do
+            I18n.backend.reload!
+          end
+
+          sub_test_case ':de' do
+            setup do
+              I18n.locale = :de
+              I18n.backend.store_translations(:de,
+                helpers: {
+                  page_entries_info: {
+                    one_page: {
+                      display_entries: {
+                        one: "Displaying <b>1</b> %{entry_name}",
+                        other: "Displaying <b>all %{count}</b> %{entry_name}"
+                      }
+                    },
+                    more_pages: {
+                      display_entries: "Displaying %{entry_name} <b>%{first}&nbsp;-&nbsp;%{last}</b> of <b>%{total}</b> in total"
+                    }
+                  }
+                }
+              )
+            end
+
+            test 'with default entry name' do
+              users = User.page(1).per(50)
+              assert_equal 'Displaying <b>all 50</b> Benutzer', view.page_entries_info(users,  entry_name: 'Benutzer')
+            end
+
+            test 'the last page with default entry name' do
+              users = User.page(5).per(10)
+              assert_equal 'Displaying Benutzer <b>41&nbsp;-&nbsp;50</b> of <b>50</b> in total', view.page_entries_info(users,  entry_name: 'Benutzer')
+            end
+          end
+        end
+
+        sub_test_case ':fr' do
+          setup do
+            I18n.locale = :fr
+            ActiveSupport::Inflector.inflections(:fr) do |inflect|
+              inflect.plural(/$/, 's')
+              inflect.singular(/s$/, '')
+            end
+            I18n.backend.store_translations(:fr,
+              helpers: {
+                page_entries_info: {
+                  one_page: {
+                    display_entries: {
+                      one: "Displaying <b>1</b> %{entry_name}",
+                      other: "Displaying <b>all %{count}</b> %{entry_name}"
+                    }
+                  },
+                  more_pages: {
+                    display_entries: "Displaying %{entry_name} <b>%{first}&nbsp;-&nbsp;%{last}</b> of <b>%{total}</b> in total"
+                  }
+                }
+              }
+            )
+          end
+
+          sub_test_case 'having 1 entry' do
+            setup do
+              User.delete_all
+              User.create! name: 'user1'
+            end
+
+            test 'with default entry name' do
+              users = User.page(1).per(25)
+              assert_equal 'Displaying <b>1</b> utilisateur', view.page_entries_info(users,  entry_name: 'utilisateur')
+            end
+          end
+
+          test 'having multiple entries with default entry name' do
+            users = User.page(1).per(50)
+            assert_equal 'Displaying <b>all 50</b> utilisateurs', view.page_entries_info(users,  entry_name: 'utilisateur')
+          end
+
+          test 'the last page with default entry name' do
+            users = User.page(5).per(10)
+            assert_equal 'Displaying utilisateurs <b>41&nbsp;-&nbsp;50</b> of <b>50</b> in total', view.page_entries_info(users,  entry_name: 'utilisateur')
+          end
+        end
       end
 
       sub_test_case 'on a model with namespace' do
@@ -353,7 +483,6 @@ if defined?(::Rails::Railtie) && defined?(::ActionView)
         31.times {|i| User.create! name: "user#{i}"}
       end
 
-
       test 'the first page' do
         users = User.page(1).per(10)
         html = view.rel_next_prev_link_tags users, params: {controller: 'users', action: 'index'}
@@ -397,6 +526,11 @@ if defined?(::Rails::Railtie) && defined?(::ActionView)
         users = User.page(2).per(1)
         assert_nil view.path_to_next_page(users, params: {controller: 'users', action: 'index'})
       end
+
+      test 'format option' do
+        users = User.page(1).per(1)
+        assert_equal '/users.turbo_stream?page=2', view.path_to_next_page(users, params: {controller: 'users', action: 'index', format: :turbo_stream})
+      end
     end
 
     sub_test_case '#path_to_prev_page' do
@@ -416,7 +550,44 @@ if defined?(::Rails::Railtie) && defined?(::ActionView)
 
       test 'the last page' do
         users = User.page(3).per(1)
-        assert_equal'/users?page=2', view.path_to_prev_page(users, params: {controller: 'users', action: 'index'})
+        assert_equal '/users?page=2', view.path_to_prev_page(users, params: {controller: 'users', action: 'index'})
+      end
+    end
+
+    sub_test_case '#next_page_url' do
+      setup do
+        2.times {|i| User.create! name: "user#{i}"}
+      end
+
+      test 'the first page' do
+        users = User.page(1).per(1)
+        assert_equal 'http://test.host/users?page=2', view.next_page_url(users, params: {controller: 'users', action: 'index'})
+      end
+
+      test 'the last page' do
+        users = User.page(2).per(1)
+        assert_nil view.next_page_url(users, params: {controller: 'users', action: 'index'})
+      end
+    end
+
+    sub_test_case '#prev_page_url' do
+      setup do
+        3.times {|i| User.create! name: "user#{i}"}
+      end
+
+      test 'the first page' do
+        users = User.page(1).per(1)
+        assert_nil view.prev_page_url(users, params: {controller: 'users', action: 'index'})
+      end
+
+      test 'the second page' do
+        users = User.page(2).per(1)
+        assert_equal 'http://test.host/users', view.prev_page_url(users, params: {controller: 'users', action: 'index'})
+      end
+
+      test 'the last page' do
+        users = User.page(3).per(1)
+        assert_equal 'http://test.host/users?page=2', view.prev_page_url(users, params: {controller: 'users', action: 'index'})
       end
     end
   end
